import json
import hashlib
from tornado import web
import base64
import os
from cqh_file.utils import get_base64, get_md5


class HandlerList(web.RequestHandler):

    def post(self):
        """
        获取获取所有文件名以及md5
        """
        target_dir = self.settings['dir']
        info_d = {}
        for name in os.listdir(target_dir):
            file_path = os.path.join(target_dir, name)
            if os.path.isfile(file_path):
                info_d[name] = get_md5(file_path)
        self.write(json.dumps(info_d, ensure_ascii=False, indent=2))


class HandlerDownload(web.RequestHandler):
    def web_write(self, d):
        self.write(json.dumps(d, ensure_ascii=False, indent=2))

    def post(self):
        d = json.loads(self.request.body)
        name = d['name']
        target_dir = self.settings['dir']
        file_path = os.path.join(target_dir, name)
        ret = {
            "code": 0,
            "data": None,
            "msg": ""
        }
        if not os.path.exists(file_path):
            ret['code'] = 1
            ret['msg'] = "not exists"
            self.web_write(ret)
            return
        if not os.path.isfile(file_path):
            ret.update({"code": 2, 'msg': "is not file"})
            self.web_write(ret)
            return
        ret['data'] = get_base64(file_path)
        self.web_write(ret)


def create_app(port, dir, timeout):
    from tornado import web, ioloop
    app = web.Application(handlers=[
        ("/list", HandlerList),
        ("/download", HandlerDownload),
    ], dir=dir)
    server = app.listen(port, xheaders=True)
    loop: ioloop.IOLoop = ioloop.IOLoop.current()
    loop.start()



import base64
import datetime
import time
import json
import os
import requests
import click

from cqh_file.utils import get_md5


class ClientLoop(object):
    def __init__(self, url, dir, sleep):
        self.url = url
        self.dir = dir
        self.sleep = sleep
        self.session = requests.Session()

    def path_url(self, url_path):
        return "{}/{}".format(self.url, url_path.lstrip("/"))

    def request_url(self, url, **kwargs):
        res = self.session.post(url, **kwargs)
        return res

    def read_serve_list(self):
        url = self.path_url("/list")
        res = self.request_url(url, json={})
        click.echo("serve, status_code:{} text:{}".format(res.status_code, res.text))
        j = res.json()
        with open(os.path.join(self.dir, ".serve.json"), 'w', encoding='utf-8') as f:
            f.write(json.dumps(j, ensure_ascii=False, indent=2))
        return j
    
    def check_res(self, res, prefix=""):
        status_code = res.status_code
        if status_code != 200:
            raise ValueError("{} status code error {}".format(prefix, status_code))
        

    def read_client_md5_value(self):
        d = {}
        for name in os.listdir(self.dir):
            file_path = os.path.join(self.dir, name)
            if os.path.exists(file_path) and os.path.isfile(file_path) and name[0] != ".":
                d[name] = get_md5(file_path)
        with open(os.path.join(self.dir, ".client.json"), 'w', encoding='utf-8') as f:
            f.write(json.dumps(d, ensure_ascii=False, indent=2))
        return d

    def loop(self):
        while 1:
            self.run_once()
            click.echo("="*80)
            click.echo("download once complete".center(80, " "))
            click.echo("="*80)
            click.echo("sleep {}, {}".format(self.sleep, datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")))
            time.sleep(self.sleep)

    def run_once(self):
        j = self.read_serve_list()
        client_d = self.read_client_md5_value()
        # 遍历所有的名字,检查需不需要下载,然后下载就好了
        for name, md5_value in j.items():
            if name not in client_d:
                self.download(name, client_d)
                continue
            if client_d[name] != md5_value:
                self.download(name, client_d)
        self.read_client_md5_value()
    
    def download(self, name,d):
        url = self.path_url("/download")
        res = self.request_url(url, json={"name": name})
        click.echo("download name:{}, cost:{}".format(name, res.elapsed.total_seconds()))
        j = res.json()
        if res.status_code != 200 or j['code']!=0:
            click.echo("download error {}".format(res.status_code, res.text))
            return
        base64_str = j['data']
        raw_data = base64.b64decode(base64_str)
        with open(os.path.join(self.dir, name), 'wb') as f:
            f.write(raw_data)




import setuptools


name = 'cqh_file'
long_description = """信息
====================================================

command
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

serve
-------------------------------------------------

opts:

* ``port`` ,  端口,默认是8081
* ``dir`` , 要目录
* ``timeout``, 默认值为60s, 内存缓存的超时时间

eg:

    sudo /home/vagrant/envs/default/bin/cqh_file --port=8081 --dir=/www/backup/path


client
-------------------------------------------------------

目的是为了下载服务器的dir

opts:

* ``url``, 地址
* ``dir``, 本地保存的目录
* ``sleep``,  睡眠时间,默认是300s



eg::

    cqh_file client --url='http://127.0.0.1:8081' --dir=/tmp

    python -m cqh_file client --url="http://192.168.146.129:8081" --dir="D:\\backup"

"""



version = "0.0.4"

setuptools.setup(
    name=name,  # Replace with your own username
    version=version,
    author="chenqinghe",
    author_email="1832866299@qq.com",
    description="cqh utils function",
    long_description=long_description,
    long_description_content_type='',
    url="https://github.com/chen19901225/cqh_util",
    packages=setuptools.find_packages(),
    install_requires=[
        "click",
        "tornado",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
    ],
    entry_points={
        "console_scripts": [
            "cqh_file=cqh_file.run:cli",
        ],
    },
    python_requires='>=3.6',
    include_package_data=True
)